#include <stdio.h>
#include <stdlib.h>
#include <check.h>

#include "hw3.h"


/** PUBLIC TESTS **/

START_TEST (test_palindrome_empty)
{ ck_assert_int_eq(is_palindrome(""), true); }
END_TEST

START_TEST (test_palindrome_char)
{ ck_assert_int_eq(is_palindrome("x"), true); }
END_TEST

START_TEST (test_palindrome_2char)
{ ck_assert_int_eq(is_palindrome("aa"), true); }
END_TEST

START_TEST (test_palindrome_simple)
{ ck_assert_int_eq(is_palindrome("aba"), true); }
END_TEST

START_TEST (test_palindrome_space_period)
{ ck_assert_int_eq(is_palindrome("Race car."), true); }
END_TEST

START_TEST (test_not_palindrome_simple)
{ ck_assert_int_eq(is_palindrome("ab"), false); }
END_TEST

START_TEST (test_not_palindrome_space)
{ ck_assert_int_eq(is_palindrome("a ba"), true); }
END_TEST

START_TEST (test_not_palindrome_longer)
{ ck_assert_int_eq(is_palindrome("this is not a palindrome"), false); }
END_TEST

START_TEST (test_slnode_count_present)
{
  sllist_t my_list;
  sllist_init(&my_list);
  sllist_addfirst(&my_list, 1);
  sllist_addfirst(&my_list, 3);
  sllist_addfirst(&my_list, 3);
  ck_assert_int_eq(sllist_count(&my_list, 3), 2);
  sllist_free(&my_list);
}
END_TEST

START_TEST (test_slnode_count_notpresent)
{
    sllist_t my_list;
    sllist_init(&my_list);
    sllist_addfirst(&my_list, 1);
    sllist_addfirst(&my_list, 2);
    sllist_addfirst(&my_list, 3);
    ck_assert_int_eq(sllist_count(&my_list, 7), 0);
    sllist_free(&my_list);
}
END_TEST

START_TEST (test_slnode_find_present)
{
    sllist_t my_list;
    sllist_init(&my_list);
    sllist_addfirst(&my_list, 3);
    sllist_addfirst(&my_list, 3);
    sllist_addfirst(&my_list, 1);
    ck_assert_int_eq(sllist_find(&my_list, 3), 1);
    sllist_free(&my_list);
}
END_TEST

START_TEST (test_slnode_find_notpresent)
{
    sllist_t my_list;
    sllist_init(&my_list);
    sllist_addfirst(&my_list, 1);
    sllist_addfirst(&my_list, 2);
    sllist_addfirst(&my_list, 3);
    ck_assert_int_eq(sllist_find(&my_list, 7), -1);
    sllist_free(&my_list);
}
END_TEST

START_TEST (test_slnode_addlast)
{
    sllist_t my_list;
    sllist_init(&my_list);
    sllist_addfirst(&my_list, 1);
    sllist_addfirst(&my_list, 2);
    sllist_addfirst(&my_list, 3);
    sllist_addlast(&my_list, 7);
    ck_assert_int_eq(sllist_find(&my_list, 7), 3);
    sllist_free(&my_list);
}
END_TEST

START_TEST (test_slnode_isequal)
{
    sllist_t my_list;
    sllist_init(&my_list);
    sllist_addfirst(&my_list, 3);
    sllist_addfirst(&my_list, 2);
    sllist_addfirst(&my_list, 1);

    sllist_t my_list2;
    sllist_init(&my_list2);
    sllist_addfirst(&my_list2, 3);
    sllist_addfirst(&my_list2, 2);
    sllist_addfirst(&my_list2, 1);

    ck_assert_int_eq(sllist_is_equal(&my_list, &my_list2), true);
    sllist_free(&my_list);
    sllist_free(&my_list2);
}
END_TEST

START_TEST (test_slnode_isnotequal)
{
    sllist_t my_list;
    sllist_init(&my_list);
    sllist_addfirst(&my_list, 1);
    sllist_addfirst(&my_list, 2);
    sllist_addfirst(&my_list, 3);


    sllist_t my_list2;
    sllist_init(&my_list2);
    sllist_addfirst(&my_list2, 2);
    sllist_addfirst(&my_list2, 3);
    sllist_addfirst(&my_list2, 1);

    ck_assert_int_eq(sllist_is_equal(&my_list, &my_list2), false);
    sllist_free(&my_list);
    sllist_free(&my_list2);
}
END_TEST

START_TEST (test_slnode_insert)
{
    sllist_t my_list;
    sllist_init(&my_list);
    sllist_addfirst(&my_list, 1);
    sllist_addfirst(&my_list, 2);
    sllist_addfirst(&my_list, 3);
    sllist_insert(&my_list, 7, 2);
    ck_assert_int_eq(my_list.head->next->next->data, 7);
    sllist_free(&my_list);
}
END_TEST

START_TEST (test_slnode_remove)
{
    sllist_t my_list;
    sllist_init(&my_list);
    sllist_addfirst(&my_list, 1);
    sllist_addfirst(&my_list, 2);
    sllist_addfirst(&my_list, 3);
    sllist_remove(&my_list, 2);
    ck_assert_int_eq(my_list.head->data, 3);
    ck_assert_int_eq(my_list.head->next->data, 1);
    ck_assert_ptr_eq(my_list.head->next->next, NULL);
    sllist_free(&my_list);
}
END_TEST

/** GENERIC TEST SUITE CODE **/

Suite * test_suite(void)
{
    Suite *s;
    TCase *tc_core;
    s = suite_create("Default");
    tc_core = tcase_create("Core");

    tcase_add_test(tc_core, test_palindrome_empty);
    tcase_add_test(tc_core, test_palindrome_char);
    tcase_add_test(tc_core, test_palindrome_2char);
    tcase_add_test(tc_core, test_palindrome_simple);
    tcase_add_test(tc_core, test_palindrome_space_period);
    tcase_add_test(tc_core, test_not_palindrome_simple);
    tcase_add_test(tc_core, test_not_palindrome_space);
    tcase_add_test(tc_core, test_not_palindrome_longer);

    tcase_add_test(tc_core, test_slnode_count_present);
    tcase_add_test(tc_core, test_slnode_count_notpresent);
    tcase_add_test(tc_core, test_slnode_find_present);
    tcase_add_test(tc_core, test_slnode_find_notpresent);
    tcase_add_test(tc_core, test_slnode_addlast);
    tcase_add_test(tc_core, test_slnode_isequal);
    tcase_add_test(tc_core, test_slnode_isnotequal);
    tcase_add_test(tc_core, test_slnode_insert);
    tcase_add_test(tc_core, test_slnode_remove);

    suite_add_tcase(s, tc_core);
    return s;
}

int run_testsuite()
{
    int number_failed;
    Suite *s;
    SRunner *sr;

    s = test_suite();
    sr = srunner_create(s);

    srunner_run_all(sr, CK_NORMAL);
    number_failed = srunner_ntests_failed(sr);
    srunner_free(sr);

    if (number_failed == 0) {
        printf("SUCCESS: All current tests passed!\n");
    } else {
        printf("FAILURE: At least one test failed or crashed.\n");
    }

    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}

int main(int argc, char* argv[])
{
    return run_testsuite();
}
